# coding: utf-8

"""
    PKS

    PKS API  # noqa: E501

    OpenAPI spec version: 1.1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class Usage(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'owner': 'str',
        'totals': 'UsageTotals',
        'clusters': 'list[ClusterUsage]'
    }

    attribute_map = {
        'owner': 'owner',
        'totals': 'totals',
        'clusters': 'clusters'
    }

    def __init__(self, owner=None, totals=None, clusters=None):  # noqa: E501
        """Usage - a model defined in Swagger"""  # noqa: E501

        self._owner = None
        self._totals = None
        self._clusters = None
        self.discriminator = None

        self.owner = owner
        self.totals = totals
        self.clusters = clusters

    @property
    def owner(self):
        """Gets the owner of this Usage.  # noqa: E501


        :return: The owner of this Usage.  # noqa: E501
        :rtype: str
        """
        return self._owner

    @owner.setter
    def owner(self, owner):
        """Sets the owner of this Usage.


        :param owner: The owner of this Usage.  # noqa: E501
        :type: str
        """
        if owner is None:
            raise ValueError("Invalid value for `owner`, must not be `None`")  # noqa: E501

        self._owner = owner

    @property
    def totals(self):
        """Gets the totals of this Usage.  # noqa: E501


        :return: The totals of this Usage.  # noqa: E501
        :rtype: UsageTotals
        """
        return self._totals

    @totals.setter
    def totals(self, totals):
        """Sets the totals of this Usage.


        :param totals: The totals of this Usage.  # noqa: E501
        :type: UsageTotals
        """
        if totals is None:
            raise ValueError("Invalid value for `totals`, must not be `None`")  # noqa: E501

        self._totals = totals

    @property
    def clusters(self):
        """Gets the clusters of this Usage.  # noqa: E501


        :return: The clusters of this Usage.  # noqa: E501
        :rtype: list[ClusterUsage]
        """
        return self._clusters

    @clusters.setter
    def clusters(self, clusters):
        """Sets the clusters of this Usage.


        :param clusters: The clusters of this Usage.  # noqa: E501
        :type: list[ClusterUsage]
        """
        if clusters is None:
            raise ValueError("Invalid value for `clusters`, must not be `None`")  # noqa: E501

        self._clusters = clusters

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Usage, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Usage):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# container-service-extension
# Copyright (c) 2019 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

import container_service_extension.exceptions as e
import container_service_extension.operation_context as ctx
import container_service_extension.request_handlers.request_utils as req_utils
from container_service_extension.shared_constants import RequestKey
from container_service_extension.telemetry.constants import CseOperation
from container_service_extension.telemetry.constants import OperationStatus
from container_service_extension.telemetry.telemetry_handler \
    import record_user_action
from container_service_extension.telemetry.telemetry_handler \
    import record_user_action_telemetry


@record_user_action_telemetry(CseOperation.SYSTEM_INFO)
def system_info(request_data, op_ctx: ctx.OperationContext):
    """Request handler for system info operation.

    :return: Dictionary with system info data.
    """
    # TODO: circular dependency with request_processor.py
    import container_service_extension.service as service
    return service.Service().info(
        get_sysadmin_info=op_ctx.client.is_sysadmin())


def system_update(request_data, op_ctx: ctx.OperationContext):
    """Request handler for system update operation.

    :return: Dictionary with system update status.
    """
    required = [
        RequestKey.SERVER_ACTION
    ]
    req_utils.validate_payload(request_data, required)

    # Telemetry data preparation
    server_action = request_data.get(RequestKey.SERVER_ACTION)
    cse_operation = server_action or 'invalid server action'
    if server_action == 'enable':
        cse_operation = CseOperation.SYSTEM_ENABLE
    elif server_action == 'disable':
        cse_operation = CseOperation.SYSTEM_DISABLE
    elif server_action == 'stop':
        cse_operation = CseOperation.SYSTEM_STOP

    status = OperationStatus.FAILED
    if op_ctx.client.is_sysadmin:
        # circular dependency between request_processor.py and service.py
        import container_service_extension.service as service
        try:
            result = service.Service().update_status(
                request_data.get(RequestKey.SERVER_ACTION))
            status = OperationStatus.SUCCESS
            return result
        finally:
            record_user_action(cse_operation=cse_operation, status=status)

    record_user_action(cse_operation=cse_operation, status=status)
    raise e.UnauthorizedRequestError(
        error_message='Unauthorized to update CSE')



# container-service-extension
# Copyright (c) 2020 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

import base64
import json
import ssl
import sys

import paho.mqtt.client as mqtt

import container_service_extension.consumer.utils as utils
import container_service_extension.logger as logger

BROKER_PATH = '/messaging/mqtt'
CLIENT_ID = 'pythonMQTT'
CONNECT_PORT = 443
TRANSPORT_WSS = 'websockets'
QOS_LEVEL = 2  # No duplicate messages


class MQTTConsumer:
    def __init__(self,
                 url,
                 listen_topic,
                 respond_topic,
                 verify_ssl,
                 token,
                 client_username):
        self.url = url
        self.listen_topic = listen_topic
        self.respond_topic = respond_topic
        self.verify_ssl = verify_ssl
        self.token = token
        self.client_username = client_username
        self.fsencoding = sys.getfilesystemencoding()
        self.mqtt_client = None

    def connect(self):
        def on_connect(mqtt_client, userdata, flags, rc):
            logger.SERVER_LOGGER.info(f'MQTT mqtt_client connected with '
                                      f'result code {rc} and flags {flags}')
            mqtt_client.subscribe(self.listen_topic, qos=QOS_LEVEL)

        def on_message(mqtt_client, userdata, msg):
            reply_body, status_code, request_id = utils.get_response_fields(
                msg=msg,
                fsencoding=self.fsencoding,
                is_mqtt=True)

            response_json = {
                "type": "API_RESPONSE",
                "headers": {
                    "requestId": request_id,
                },
                "httpResponse": {
                    "statusCode": status_code,
                    "headers": {
                        "Content-Type": "application/json",
                        'Content-Length': len(reply_body)
                    },
                    "body": base64.b64encode(reply_body.encode()).
                    decode(sys.getfilesystemencoding())
                }
            }

            pub_ret = mqtt_client.publish(topic=self.respond_topic,
                                          payload=json.dumps(response_json),
                                          qos=QOS_LEVEL,
                                          retain=False)
            logger.SERVER_LOGGER.info(f"pub_ret (rc, msg_id): {pub_ret}")

        def on_subscribe(mqtt_client, userdata, msg_id, given_qos):
            logger.SERVER_LOGGER.info(f'MQTT mqtt_client subscribed with '
                                      f'given_qos: {given_qos}')

        def on_disconnect(mqtt_client, userdata, rc):
            logger.SERVER_LOGGER.info(f'MQTT disconnect with reason: {rc}')

        self.mqtt_client = mqtt.Client(client_id=CLIENT_ID,
                                       transport=TRANSPORT_WSS)
        self.mqtt_client.username_pw_set(username=self.client_username,
                                         password=self.token)
        cert_req = ssl.CERT_REQUIRED if self.verify_ssl else ssl.CERT_NONE
        self.mqtt_client.tls_set(cert_reqs=cert_req)
        self.mqtt_client.ws_set_options(path=BROKER_PATH)

        # Setup callbacks
        self.mqtt_client.on_connect = on_connect
        self.mqtt_client.on_message = on_message
        self.mqtt_client.on_disconnect = on_disconnect
        self.mqtt_client.on_subscribe = on_subscribe

        try:
            self.mqtt_client.connect(self.url, port=CONNECT_PORT)
        except Exception as e:
            logger.SERVER_LOGGER.error(f'connection error: {e}')
            raise e
        self.mqtt_client.loop_forever()

    def run(self):
        self.connect()

    def stop(self):
        if self.mqtt_client:
            self.mqtt_client.disconnect()



# container-service-extension
# Copyright (c) 2020 VMware, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-2-Clause

import base64
import json
import traceback

import requests

from container_service_extension.exceptions import CseRequestError
from container_service_extension.logger import SERVER_LOGGER as LOGGER
import container_service_extension.request_processor as request_processor
from container_service_extension.shared_constants import RESPONSE_MESSAGE_KEY


def get_response_fields(msg, fsencoding, is_mqtt):
    """Get the reply body, status code, and request id from the message."""
    try:
        # Parse the message
        if is_mqtt:
            payload_json = json.loads(msg.payload.decode(fsencoding))
            http_req_json = json.loads(base64.b64decode(
                payload_json['httpRequest']))
            request_id = payload_json["headers"]["requestId"]
            msg_json = http_req_json['message']
        else:
            msg_json = json.loads(msg.decode(fsencoding))[0]
            request_id = msg_json['id']

        result = request_processor.process_request(msg_json)
        status_code = result['status_code']
        reply_body = json.dumps(result['body'])
    except Exception as e:
        if isinstance(e, CseRequestError):
            status_code = e.status_code
        else:
            status_code = requests.codes.internal_server_error
        reply_body = json.dumps({RESPONSE_MESSAGE_KEY: str(e)})

        tb = traceback.format_exc()
        LOGGER.error(tb)

    return reply_body, status_code, request_id
